@@ -3,14 +3,17 @@ from .util import *
 
 class Command:
     resolution_mode = 0
+    steps = 0
+    maxVolum = 0.0
+    syringeStroke = 0
 
     def __init__(self, type: PSDTypes):
         self.type = type
 
     """
-    Zx - Initialize PSD/4, Assign Valve Output to Right
-    Yx - Initialize PSD/4, Assign Valve Output to Left
-    Wx - Initialize PSD/4, Configure for No Valve
+    Zx - Initialize PSD, Assign Valve Output to Right
+    Yx - Initialize PSD, Assign Valve Output to Left
+    Wx - Initialize PSD, Configure for No Valve
     """
     def initialize(self, drive: str, value=0):
         cmd: str = ''
@@ -41,6 +44,189 @@ class Command:
         Syringe Commands
     """
 
+    # This script can select the type of movement between absolute or relative pickup or dispense, also return steps or backoffSteps
+    # using first parameter as selection mode and second parameter as fluid volume requested per second instead of motor steps/seconds
+    # Parameters:
+    # - typeOf parameter: is the type of movement,
+    #                   accepted values:
+    #                       absoluteMovement = 'absolute'
+    #                       relativePickup = 'pickup'
+    #                       relativeDispense = 'dispense'
+    #                       returnSteps = 'return'
+    #                       backoffSteps = 'backoff'
+    # - volume parameter: is volume required per second, volume value is measured in micro liters
+    #
+    def syringeMovement(self, typeOf: str, volume: float):
+        cmd: str = ''
+        if typeOf == SyringeMovement.absoluteMovement.value:
+            cmd += 'A'
+        elif typeOf == SyringeMovement.relativePickup.value:
+            cmd += 'P'
+        elif typeOf == SyringeMovement.relativeDispense.value:
+            cmd += 'D'
+        elif typeOf == SyringeMovement.returnSteps.value:
+            cmd += 'K'
+        elif typeOf == SyringeMovement.backoffSteps.value:
+            cmd += 'k'
+        else:
+            print("Invalid value \"" + str(typeOf) + "\" for type of movement!")
+            cmd = 'cmdError'
+
+        value: int = self.volumeToSteps(volume)
+        if self.checkIntervalCorrectness(value, typeOf) == True:
+            cmd += str(value)
+        else:
+            print("Invalid value \"" + str(volume) + "\" for volume!")
+            cmd = 'cmdError'
+        return cmd
+
+    # This script can set start, stop and maximum velocity using as parameter the fluid volume requested per second
+    # instead of motor steps/seconds
+    # Parameters:
+    # - typeOf parameter: is the type of velocity,
+    #                   accepted values:
+    #                       maxVelocity = 'max'
+    #                       startVelocity = 'start'
+    #                       stopVelocity = 'stop'
+    # - volume parameter: is volume required per second, volume value is measured in micro liters
+    #
+    def velocityConfiguration(self, typeOf: str, volume: float):
+        cmd: str = ''
+        if typeOf == VelocityTypes.maxVelocity.value:
+            cmd += 'V'
+        elif typeOf == VelocityTypes.startVelocity.value:
+            cmd += 'v'
+        elif typeOf == VelocityTypes.stopVelocity.value:
+            cmd += 'c'
+        else:
+            print("Invalid value \"" + str(typeOf) + "\" for type of velocity!")
+            cmd = 'cmdError'
+            return cmd
+
+        parameterV: int = self.parameterVCalculation(volume)
+        if self.checkParameterV(parameterV, typeOf) == True:
+            cmd += str(parameterV)
+        else:
+            print("Invalid value \"" + str(volume) + "\" for volume!")
+            cmd = 'cmdError'
+        return cmd
+
+    # helper method that checks if the value is in the correct range
+    def checkValueInInterval(self, value: int, valueST: int, valueHG: int):
+        bVal: bool = False
+        # if standard mode
+        if self.resolution_mode == 0 and 0 <= value <= valueST:
+            bVal = True
+        elif self.resolution_mode == 1 and 0 <= value <= valueHG:
+            bVal = True
+        else:
+            print("Error - value out of range !!!")
+        return bVal
+
+    # calculation of flow rate for parameter "u"
+    def calculateParameterU(self, volRequested: float):
+        result: float = 0
+        if (self.syringeStroke == 192000 or self.syringeStroke == 384000) and \
+                ( 0.0 <= volRequested <= self.maxVolum and self.maxVolum != 0 ):
+            result = (volRequested * self.syringeStroke / self.maxVolum)
+
+        return (int)(result)
+
+    # calculation of flow rate for parameter "u"
+    def parameterVCalculation(self, volRequested: float):
+        result: float = 0
+        if 0.0 <= volRequested <= self.maxVolum and self.maxVolum != 0:
+            result = (volRequested * self.syringeStroke / self.maxVolum)
+
+        if self.syringeStroke == 192000 or self.syringeStroke == 384000:
+            result /= 4
+
+        return (int)(result)
+
+    # helper method used in velocityConfiguration method
+    # It has the goal to check if value calculated by parameterVCalculation method is in the correct range
+    def checkParameterV(self, value: int, typeOf: str):
+        result: bool = False
+        if self.syringeStroke == 6000 or self.syringeStroke == 12000:
+            if typeOf == VelocityTypes.maxVelocity.value and 2 <= value <= 5800:
+                result = True
+            elif typeOf == VelocityTypes.startVelocity.value and 50 <= value <= 1000:
+                result = True
+            elif typeOf == VelocityTypes.stopVelocity.value and 50 <= value <= 2700:
+                result = True
+            else:
+                result = False
+        elif self.syringeStroke == 192000 or self.syringeStroke == 384000:
+            if typeOf == VelocityTypes.maxVelocity.value and 2 <= value <= 3400:
+                result = True
+            elif typeOf == VelocityTypes.startVelocity.value and 50 <= value <= 800:
+                result = True
+            elif typeOf == VelocityTypes.stopVelocity.value and 50 <= value <= 1700:
+                result = True
+            else:
+                result = False
+        else:
+            print("Invalid type!")
+        return result
+
+    # helper method that checks if the value is in the correct range for each movement type
+    def checkIntervalCorrectness(self, value: int, syringeMov: str):
+        result: bool = False
+        if self.syringeStroke == 6000:
+            if syringeMov == SyringeMovement.absoluteMovement.value or \
+                    syringeMov == SyringeMovement.relativePickup.value or \
+                    syringeMov == SyringeMovement.relativeDispense.value:
+                result = self.checkValueInInterval(value, 3000, 24000)
+            elif syringeMov == SyringeMovement.returnSteps.value:
+                result = self.checkValueInInterval(value, 100, 800)
+            elif syringeMov == SyringeMovement.backoffSteps.value:
+                result = self.checkValueInInterval(value, 200, 1600)
+            else:
+                result = False
+        elif self.syringeStroke == 12000:
+            if syringeMov == SyringeMovement.absoluteMovement.value or \
+                    syringeMov == SyringeMovement.relativePickup.value or \
+                    syringeMov == SyringeMovement.relativeDispense.value:
+                result = self.checkValueInInterval(value, 6000, 48000)
+            elif syringeMov == SyringeMovement.returnSteps.value:
+                result = self.checkValueInInterval(value, 100, 800)
+            elif syringeMov == SyringeMovement.backoffSteps.value:
+                result = self.checkValueInInterval(value, 200, 1600)
+            else:
+                result = False
+        elif self.syringeStroke == 192000:
+            if syringeMov == SyringeMovement.absoluteMovement.value or \
+                    syringeMov == SyringeMovement.relativePickup.value or \
+                    syringeMov == SyringeMovement.relativeDispense.value:
+                result = self.checkValueInInterval(value, 192000, 192000)
+            elif syringeMov == SyringeMovement.returnSteps.value:
+                result = self.checkValueInInterval(value, 6400, 6400)
+            elif syringeMov == SyringeMovement.backoffSteps.value:
+                result = self.checkValueInInterval(value, 12800, 12800)
+            else:
+                result = False
+        elif self.syringeStroke == 384000:
+            if syringeMov == SyringeMovement.absoluteMovement.value or \
+                    syringeMov == SyringeMovement.relativePickup.value or \
+                    syringeMov == SyringeMovement.relativeDispense.value:
+                result = self.checkValueInInterval(value, 384000, 384000)
+            elif syringeMov == SyringeMovement.returnSteps.value:
+                result = self.checkValueInInterval(value, 6400, 6400)
+            elif syringeMov == SyringeMovement.backoffSteps.value:
+                result = self.checkValueInInterval(value, 12800, 12800)
+            else:
+                result = False
+        else:
+            print("Invalid interval!")
+        return result
+
+    # helper method that translates volume in motor steps
+    def volumeToSteps(self, volRequested: float):
+        result: int = 0
+        if 0.0 <= volRequested <= self.maxVolum and self.maxVolum != 0:
+            result = (int)(volRequested * self.steps / self.maxVolum)
+        return result
+
     # z - Set Counter Position
     def setCounterPosition(self):
         return 'z'
